@@ -18,19 +18,20 @@
 #  FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 #  DEALINGS IN THE SOFTWARE.
 """Base class used by init command."""
+from __future__ import annotations
 
 import logging
 
 from molecule.command import base
-from molecule.command.init import role, scenario
+from molecule.command.init import scenario
+
 
 LOG = logging.getLogger(__name__)
 
 
-@base.click_group_ex()  # type: ignore
-def init():  # pragma: no cover
-    """Initialize a new role or scenario."""
+@base.click_group_ex()
+def init() -> None:  # pragma: no cover
+    """Initialize a new scenario."""
 
 
-init.add_command(role.role)
 init.add_command(scenario.scenario)
